@@ -1,6 +1,6 @@
 ##
 ## R package reda by Wenjie Wang, Haoda Fu, and Jun Yan
-## Copyright (C) 2015-2021
+## Copyright (C) 2015-2022
 ##
 ## This file is part of the R package reda.
 ##
@@ -61,20 +61,13 @@ setMethod(
         n_xx <- control$length.out
 
         ## baseline rate basis matrix
-        spline <- object@spline$spline
-        if (identical(spline, "bSplines")) {
-            iMat <- splines2::ibs(x = gridTime, knots = knots,
-                                  degree = degree, intercept = TRUE,
-                                  Boundary.knots = Boundary.knots)
-            bMat <- attr(iMat, "bsMat")
-        } else if (identical(spline, "mSplines")) {
-            iMat <- splines2::iSpline(x = gridTime, knots = knots,
-                                      degree = degree, intercept = TRUE,
-                                      Boundary.knots = Boundary.knots)
-            bMat <- attr(iMat, "msMat")
-        } else
-            stop("Unknown splines.")
-
+        iMat <- splines2::mSpline(x = gridTime,
+                                  knots = knots,
+                                  degree = degree,
+                                  intercept = TRUE,
+                                  integral = TRUE,
+                                  Boundary.knots = Boundary.knots,
+                                  periodic = object@spline$periodic)
         ## estimated MCF
         estMcf <- as.vector(iMat %*% alpha)
 
@@ -194,11 +187,11 @@ rateReg_mcf_control <- function(grid, length.out = 1e3, from, to, ...,
     } else {
         grid <- seq.int(from = from, to = to, length.out = length.out)
     }
-    if (min(grid) < Boundary.knots_[1] || max(grid) > Boundary.knots_[2])
-        stop(wrapMessages(
-            "The 'grid' specified must be within",
-            "the coverage of the boundary knots."
-        ), call. = FALSE)
+    ## if (min(grid) < Boundary.knots_[1] || max(grid) > Boundary.knots_[2])
+    ##     stop(wrapMessages(
+    ##         "The 'grid' specified must be within",
+    ##         "the coverage of the boundary knots."
+    ##     ), call. = FALSE)
 
     ## return
     list(grid = grid, length.out = length.out, from = from, to = to)
